import { NotificationStructure } from '@genesislcap/foundation-notifications';
import { FoundationElement } from '@microsoft/fast-foundation';
export declare const foundationDialogShadowOptions: ShadowRootInit;
export declare const defaultDialogConfig: {};
export declare abstract class DialogElement extends FoundationElement {
    /**
     * A reference to the dialog element
     * in the component Shadow DOM.
     */
    mainElement: HTMLDialogElement;
    notification: NotificationStructure;
    notificationChanged(): void;
    position: 'centre' | 'left' | 'right';
    /**
     * Whether or not to show the close icon
     */
    showCloseIcon: boolean;
    /**
     * Callback to run *before* HTMLDialogElement is shown
     */
    onShowCallback: () => void;
    /**
     * Callback to run *after* HTMLDialogElement is closed
     */
    onCloseCallback: () => void;
    connectedCallback(): void;
    /**
     * Determine if the dialog is showing or closed.
     */
    get open(): boolean;
    /**
     * Show the HTMLDialogElement and its contents
     */
    abstract show(): void;
    /**
     * Remove the HTMLDialogElement from display
     */
    close(): void;
    /**
     * Dismiss the Dialog from display and calls the dismissing action.
     */
    dismiss(): void;
    /**
     * Dismiss the Dialog from display.
     */
    closeDialog(): void;
}
/**
 * @tagname %%prefix%%-dialog
 */
export declare class Dialog extends DialogElement {
    show(): void;
}
/**
 * The Foundation Dialog
 *
 * @public
 * @remarks
 * HTML Element: \<foundation-dialog\>
 */
export declare const foundationDialog: (overrideDefinition?: import("@microsoft/fast-foundation").OverrideFoundationElementDefinition<{
    baseName: string;
    styles: import("@microsoft/fast-element").ElementStyles;
    template: import("@microsoft/fast-element").ViewTemplate<Dialog, any>;
    shadowOptions: ShadowRootInit;
}>) => import("@microsoft/fast-foundation").FoundationElementRegistry<{
    baseName: string;
    styles: import("@microsoft/fast-element").ElementStyles;
    template: import("@microsoft/fast-element").ViewTemplate<Dialog, any>;
    shadowOptions: ShadowRootInit;
}, typeof Dialog>;
//# sourceMappingURL=dialog.d.ts.map