import { Connect, Datasource, DatasourceOptions, FilteredDataServerResult, RequestServerResult } from '@genesislcap/foundation-comms';
import { FoundationElement } from '@microsoft/fast-foundation';
import { BehaviorSubject, Subscription } from 'rxjs';
export declare abstract class BaseDatasource extends FoundationElement {
    connect: Connect;
    datasource: Datasource;
    withGridInit: boolean;
    criteria: string;
    criteriaChanged(oldCriteria: string, newCriteria: string): void;
    data: any[];
    fields: string;
    isSnapshot: boolean;
    maxRows: number;
    maxView: number;
    orderBy: string;
    request: any;
    resourceName: string;
    reverse: boolean;
    rowId: string;
    /**
     * Enabled by default, will format UNIX timestamp fields to readable [DATE|DATETIME] values.
     * @public
     */
    withTimestampFormatting: boolean;
    set datasourceConfig(config: DatasourceOptions);
    requiresFullDataRefresh: boolean;
    dataSub: Subscription;
    private sourceRef;
    rowsSyncedWithStream: Map<string, any>;
    dataSubWasLoggedOff: boolean;
    private criteriaFromFilters;
    update: BehaviorSubject<Map<string, string>>;
    disconnectedCallback(): void;
    reset(): void;
    abstract clearData(): void;
    abstract syncComponentData(snapshotData?: any[]): void;
    abstract handleStreamInserts(insertedRows: any[]): void;
    abstract handleStreamDeletes(deletedRows: any[]): void;
    abstract handleStreamUpdates(updatedRows: any[]): void;
    fetchGenesisData(withFullInit?: boolean): Promise<void>;
    initDatasource(): Promise<void>;
    private datasourceOptions;
    private handleStreamUpdate;
    applyRequestServerData(requestServerResult: RequestServerResult): void;
    applyDataserverData(dataServerResult: FilteredDataServerResult): void;
    private buildCriteria;
    setFilter(fieldName: string, newFilter: string): void;
    removeFilter(fieldName: string): void;
}
//# sourceMappingURL=base-datasource.d.ts.map